// ----------------------------------------------------------------------------
// TDTPageOrientation
// Version: 1.0. Released 12th Feb 2009.
// Author: David Tetard.
//
// Tested on BDS 2006 only, under Windows XP.
//
// Released to the public domain, no liability accepted.
// Free for commercial and non-commercial use (simply acknowledge origin
//  somewhere in about box, readme or help file).
//
// If you find a bug, drop an e-mail at ChemUlysses@Hotmail.com
// ----------------------------------------------------------------------------


Unit DTPageOrientation;

Interface

Uses Windows, Classes, Messages, Controls, Graphics, Printers;

Type
    TPageOrientationBorderStyle = (
         POBS_None,
         POBS_Flat,
         POBS_Sunken,
         POBS_Edged
    );

    TFocusedPage = (
         FP_None,
         FP_Portrait,
         FP_Landscape
    );

    TPageLayout = (
         PL_Vertical,
         PL_Horizontal
    );

    TDTPageOrientation = Class(TCustomControl)
    Private
         FFocusedPage : TFocusedPage;
         FLandscapeCaption : String;
         FLandscapeImage : TPicture;
         FLandscapeRect : TRect;
         FOnPageOrientationChanged : TNotifyEvent;
         FPageBackgroundColor : TColor;
         FPageLayout : TPageLayout;
         FPageOrientation : TPrinterOrientation;
         FPageOrientationBorderStyle : TPageOrientationBorderStyle;
         FPageSeparation : Cardinal;
         FPortraitCaption : String;
         FPortraitImage : TPicture;
         FPortraitRect : TRect;
         FShowCaptions : Boolean;

         Procedure CalculateImageRects;
         Procedure CMDialogChar(var Msg: TCMDialogChar); message CM_DIALOGCHAR;
         Procedure DoPageOrientationChanged;
         Procedure DrawFocus(Const PageRect : TRect);
         Procedure DrawPage(Const PageRect : TRect; Const Picture : TPicture;
              Const aCaption : String; Const IsSelected : Boolean);
         Procedure NotifyPictureChanged(Sender : TObject);
         Procedure SetLandscapeCaption(const Value: String);
         Procedure SetLandscapeImage(const Value: TPicture);
         Procedure SetPageBackgroundColor(const Value: TColor);
         Procedure SetPageLayout(const Value: TPageLayout);
         Procedure SetPageOrientation(const Value: TPrinterOrientation);
         Procedure SetPageOrientationBorderStyle(const Value: TPageOrientationBorderStyle);
         Procedure SetPageSeparation(const Value: Cardinal);
         Procedure SetPortraitCaption(const Value: String);
         Procedure SetPortraitImage(const Value: TPicture);
         Procedure SetShowCaptions(const Value: Boolean);
    Protected
         Procedure Paint; Override;
         Procedure WndProc(var Message: TMessage); Override;
    Public
         Constructor Create(AOwner: TComponent); Override;
         Destructor Destroy; Override;
         Property DockManager;
    Published
         // Properties:
         Property LandscapeCaption: String Read FLandscapeCaption
              Write SetLandscapeCaption;
         Property OnPageOrientationChanged: TNotifyEvent Read FOnPageOrientationChanged
              Write FOnPageOrientationChanged;
         Property PageBackgroundColor: TColor Read FPageBackgroundColor
              Write SetPageBackgroundColor;
         Property PageLayout: TPageLayout Read FPageLayout Write SetPageLayout;
         Property PageOrientation: TPrinterOrientation Read FPageOrientation
             Write SetPageOrientation;
         Property PageOrientationBorderStyle: TPageOrientationBorderStyle
             Read FPageOrientationBorderStyle Write SetPageOrientationBorderStyle;
         Property PageSeparation: Cardinal Read FPageSeparation Write SetPageSeparation;
         Property PictureLandscape : TPicture Read FLandscapeImage Write SetLandscapeImage;
         Property PicturePortrait : TPicture Read FPortraitImage Write SetPortraitImage;
         Property PortraitCaption: String Read FPortraitCaption Write SetPortraitCaption;
         Property ShowCaptions: Boolean Read FShowCaptions Write SetShowCaptions;
         // From TCustomControl:
         property Align;
         property Anchors;
         property BiDiMode;
         property Color;
         property Constraints;
         property Ctl3D;
         property UseDockManager default True;
         property DockSite;
         property DragCursor;
         property DragKind;
         property DragMode;
         property Enabled;
         property Font;
         property Padding;
         property ParentBiDiMode;
         property ParentBackground;
         property ParentColor;
         property ParentCtl3D;
         property ParentFont;
         property ParentShowHint;
         property PopupMenu;
         property ShowHint;
         property TabOrder;
         property TabStop;
         property Visible;
         property OnAlignInsertBefore;
         property OnAlignPosition;
         property OnCanResize;
         property OnClick;
         property OnConstrainedResize;
         property OnContextPopup;
         property OnDockDrop;
         property OnDockOver;
         property OnDblClick;
         property OnDragDrop;
         property OnDragOver;
         property OnEndDock;
         property OnEndDrag;
         property OnEnter;
         property OnExit;
         property OnGetSiteInfo;
         property OnMouseActivate;
         property OnMouseDown;
         property OnMouseEnter;
         property OnMouseLeave;
         property OnMouseMove;
         property OnMouseUp;
         property OnResize;
         property OnStartDock;
         property OnStartDrag;
         property OnUnDock;
    End;

Procedure Register;


Implementation

Uses Forms, Math;

{$R DTPageOrientationRes.Res}

{ TDTPageOrientation }


{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }


{ ---------------------------------------------------------------------------- }
Procedure Register;
Begin
     RegisterComponents('Added', [TDTPageOrientation]);
End;


{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }


{ ---------------------------------------------------------------------------- }
Constructor TDTPageOrientation.Create(AOwner: TComponent);
ResourceString
     ResTxt_PortraitCaption = 'Portrait';
     ResTxt_LandscapeCaption = 'Landscape';
Begin
     Inherited;

     FPageSeparation := 20;
     FFocusedPage := FP_None;
     FLandscapeCaption := ResTxt_LandscapeCaption;
     FPageBackgroundColor := clWindow;
     FOnPageOrientationChanged := Nil;
     FPageLayout := PL_Horizontal;
     FPageOrientation := poPortrait;
     FPageOrientationBorderStyle := POBS_Flat;
     FPortraitCaption := ResTxt_PortraitCaption;
     FShowCaptions := True;

     FPortraitImage := TPicture.Create;
     FPortraitImage.OnChange := NotifyPictureChanged;
     FPortraitImage.Bitmap.LoadFromResourceName(HInstance, 'DTPageOrientation_Portrait');
     FPortraitImage.Bitmap.Transparent := True;

     FLandscapeImage := TPicture.Create;
     FLandscapeImage.OnChange := NotifyPictureChanged;
     FLandscapeImage.Bitmap.LoadFromResourceName(HInstance, 'DTPageOrientation_Landscape');
     FLandscapeImage.Bitmap.Transparent := True;
End;


{ ---------------------------------------------------------------------------- }
Destructor TDTPageOrientation.Destroy;
Begin
     FPortraitImage.Free;
     FLandscapeImage.Free;

     Inherited;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.WndProc(var Message: TMessage);
Var
   Pt : TPoint;
Begin
     If Not (csDesigning In ComponentState) Then
     Begin
          Case Message.Msg Of
               WM_KILLFOCUS :
                    Invalidate;
               WM_SETFOCUS :
                    Invalidate;
               WM_LBUTTONUP : Begin
                    Pt := ScreenToClient(Mouse.CursorPos);
                    If PtInRect(FPortraitRect, Pt) Then
                         SetPageOrientation(poPortrait)
                    Else If PtInRect(FLandscapeRect, Pt) Then
                         SetPageOrientation(poLandscape);
               End;
               WM_LBUTTONDOWN, WM_LBUTTONDBLCLK : Begin
                    Pt := ScreenToClient(Mouse.CursorPos);
                    If PtInRect(FPortraitRect, Pt) Then
                    Begin
                         If FFocusedPage <> FP_Portrait Then
                              FFocusedPage := FP_Portrait;
                    End
                    Else If PtInRect(FLandscapeRect, Pt) Then
                    Begin
                         If FFocusedPage <> FP_Landscape Then
                              FFocusedPage := FP_Landscape;
                    End
                    Else FFocusedPage := FP_None;
                    SetFocus;
                    Refresh;
               End;
          End;
     End;
     Inherited WndProc(Message);
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.CMDialogChar(var Msg: TCMDialogChar);
Begin
     Inherited;

     If IsAccel(Msg.CharCode, FPortraitCaption) Then
     Begin
          SetPageOrientation(poPortrait);
          Msg.Result := 1;
     End
     Else If IsAccel(Msg.CharCode, FLandscapeCaption) Then
     Begin
          SetPageOrientation(poLandscape);
          Msg.Result := 1;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.NotifyPictureChanged(Sender: TObject);
Begin
     Invalidate;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.DrawFocus(Const PageRect: TRect);
Var
   Rect : TRect;
Begin
     // Focus:
     Rect := PageRect;
     Rect.Left := Rect.Left + 3;
     Rect.Top := Rect.Top + 3;
     Rect.Right := Rect.Right - 3;
     Rect.Bottom := Rect.Bottom - 3;
     DrawFocusRect(Canvas.Handle, Rect);
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.DrawPage(Const PageRect : TRect; Const Picture: TPicture;
     Const aCaption: String; Const IsSelected : Boolean);
Const
     SelectionOffset = 5;

   { ---------------------------------------------------------------------------- }
   Procedure DrawBorder(Var Rect : TRect);
   Begin
        Case FPageOrientationBorderStyle Of
             // POBS_None : Nothing.
             POBS_Flat : Begin
                  DrawEdge(Canvas.Handle, Rect, BDR_RAISEDOUTER,
                       BF_ADJUST Or BF_Flat Or BF_Rect);
             End;
             POBS_Sunken : Begin
                  DrawEdge(Canvas.Handle, Rect, BDR_SUNKENOUTER, BF_ADJUST Or BF_Rect);
                  Rect.Right := Rect.Right - 1;
                  Rect.Bottom := Rect.Bottom - 1;
             End;
             POBS_Edged : begin
                  Rect.Left := Rect.Left + 1;
                  Rect.Top := Rect.Top + 1;
                  DrawEdge(Canvas.Handle, Rect, EDGE_ETCHED, BF_ADJUST Or BF_Rect);
             End;
        End;
   End;

Var
   OldClipRect : TRect;
   Rect : TRect;
   Region : HRGN;
   x, y : Integer;
Begin
     // Draw background of images:
     Rect := PageRect;
     DrawBorder(Rect);
     Canvas.Brush.Style := bsSolid;
     Canvas.Brush.Color := FPageBackgroundColor;
     Canvas.Pen.Color := FPageBackgroundColor;
     Canvas.Rectangle(Rect);

     // Draw captions:
     If FShowCaptions Then
     Begin
          Canvas.Pen.Color := Font.Color;
          Canvas.Font.Assign(Font);
          Canvas.Brush.Color := Color;

          Rect := PageRect;
          Rect.Top := Rect.Bottom + 2;
          Rect.Bottom := BoundsRect.Bottom;
          DrawText(Canvas.Handle, PChar(aCaption), -1, Rect, DT_SingleLine Or DT_CENTER);
     End;

     // Border rect:
     // NB: Need that before drawing picture so that can use that rect
     //  for the clipping region.
     Rect.Left := PageRect.Left + SelectionOffset;
     Rect.Top := PageRect.Top + SelectionOffset;
     Rect.Right := PageRect.Right - SelectionOffset + 1;
     Rect.Bottom := PageRect.Bottom - SelectionOffset + 1;
     If FPageOrientationBorderStyle = POBS_Sunken Then
     Begin
          Rect.Right := Rect.Right - 1;
          Rect.Bottom := Rect.Bottom - 1;
     End;

     // Page images:
     OldClipRect := Canvas.ClipRect;
     Region := CreateRectRgn(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
     SelectClipRgn(Canvas.Handle, Region);

     x := PageRect.Left + ((PageRect.Right - PageRect.Left - Picture.Width) Div 2);
     y := PageRect.Top + ((PageRect.Bottom - PageRect.Top - Picture.Height) Div 2);
     Canvas.Draw(x, y, Picture.Bitmap);

     DeleteObject(Region);
     Region := CreateRectRgn(OldClipRect.Left, OldClipRect.Top, OldClipRect.Right,
          OldClipRect.Bottom);
     SelectClipRgn(Canvas.Handle, Region);

     // Selection:
     If IsSelected Then
     Begin
          Canvas.Pen.Width := 2;
          Canvas.Pen.Color := clHighlight;
          Canvas.Brush.Style := bsClear;
          Canvas.Rectangle(Rect);
          Canvas.Pen.Width := 1;
          Canvas.Brush.Style := bsSolid;
          Canvas.Pen.Color := clBlack;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.Paint;
Var
   Rect : TRect;
Begin
     CalculateImageRects;

     // Draw background:
     Rect := BoundsRect;
     Canvas.Brush.Style := bsSolid;
     Canvas.Brush.Color := Color;
     Canvas.Pen.Color := Color;
     Canvas.Rectangle(Rect);

     // Draw each page:
     DrawPage(FPortraitRect, FPortraitImage, FPortraitCaption, (FPageOrientation = poPortrait));
     DrawPage(FLandscapeRect, FLandscapeImage, FLandscapeCaption, (FPageOrientation = poLandscape));

     If Focused Then
          Case FFocusedPage Of
               FP_Portrait :
                    DrawFocus(FPortraitRect);
               FP_Landscape :
                    DrawFocus(FLandscapeRect);
          End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.CalculateImageRects;
Var
   CaptionHeight : Integer;
   ImageWidth : Integer;
Begin
     If FShowCaptions Then
          CaptionHeight := Max(Canvas.TextHeight(FPortraitCaption),
               Canvas.TextHeight(FLandscapeCaption)) + 2
     Else CaptionHeight := 0;

     Case FPageLayout Of
          PL_Horizontal : Begin
               ImageWidth := (ClientWidth - FPageSeparation) Div 2;

               FPortraitRect := Rect(0, 0, ImageWidth,
                    ClientHeight - CaptionHeight - 2);
               FLandscapeRect := Rect(ClientWidth - ImageWidth, 0, ClientWidth,
                    ClientHeight - CaptionHeight - 2);
          End;
          PL_Vertical : Begin
               If FShowCaptions Then
               Begin
                    ImageWidth := (Clientheight - FPageSeparation - 2 * CaptionHeight) Div 2;

                    FPortraitRect := Rect(0, 0, ClientWidth, ImageWidth);
                    FLandscapeRect := Rect(0, ClientHeight - ImageWidth - CaptionHeight,
                         ClientWidth, ClientHeight - CaptionHeight);
               End
               Else Begin
                    ImageWidth := (ClientHeight - FPageSeparation) Div 2;

                    FPortraitRect := Rect(0, 0, ClientWidth, ImageWidth);
                    FLandscapeRect := Rect(0, ClientHeight - ImageWidth, ClientWidth,
                         ClientHeight);
               End;
          End;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.DoPageOrientationChanged;
Begin
     If Assigned(FOnPageOrientationChanged) Then
          FOnPageOrientationChanged(Self);
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPageSeparation(const Value: Cardinal);
Begin
     If FPageSeparation <> Value Then
     Begin
          FPageSeparation := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetLandscapeCaption(const Value: String);
Begin
     If FLandscapeCaption <> Value Then
     Begin
          FLandscapeCaption := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetLandscapeImage(const Value: TPicture);
Begin
     FLandscapeImage.Assign(Value);
     Invalidate;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPageBackgroundColor(const Value: TColor);
Begin
     If FPageBackgroundColor <> Value Then
     Begin
          FPageBackgroundColor := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPageLayout(const Value: TPageLayout);
Begin
     If FPageLayout <> Value Then
     Begin
          FPageLayout := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPageOrientation(const Value: TPrinterOrientation);
Begin
     If FPageOrientation <> Value Then
     Begin
          FPageOrientation := Value;
          Invalidate;
          DoPageOrientationChanged;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPageOrientationBorderStyle(
  const Value: TPageOrientationBorderStyle);
Begin
     If FPageOrientationBorderStyle <> Value Then
     Begin
          FPageOrientationBorderStyle := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPortraitCaption(const Value: String);
Begin
     If FPortraitCaption <> Value Then
     Begin
          FPortraitCaption := Value;
          Invalidate;
     End;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetPortraitImage(const Value: TPicture);
Begin
     FPortraitImage.Assign(Value);
     Invalidate;
End;


{ ---------------------------------------------------------------------------- }
Procedure TDTPageOrientation.SetShowCaptions(const Value: Boolean);
Begin
     If FShowCaptions <> Value Then
     Begin
          FShowCaptions := Value;
          Invalidate;
     End;
End;


End.
